/***************************************************************************
                          analyticestimator.cpp  -  description
                             -------------------
    begin                : Thu Jul 28 2005
    copyright            : (C) 2005 by Michael Rosskopf
    email                : rosskopf@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "analyticestimator.h"

// Constructors and destructors
AnalyticEstimator::AnalyticEstimator(){
  estimate = new ModelParameters();
}
AnalyticEstimator::~AnalyticEstimator(){
  if(estimate != 0) delete estimate;
}

// getEstimate: Returns the result of the previous parameter estimation
ModelParameters* AnalyticEstimator::getEstimate(){
  return estimate;
}

// calculateEstimate: Estimates the model parameters analytically
//                     (only available for the M-normal and the M-extreme model)
void AnalyticEstimator::calculateEstimate(ProgramParameters *programParameters,DistributionMeasures *targetValues){

  if(programParameters->getSample3() == 0){

    // No use of an outgroup

    // Normal distribution
    if(programParameters->getMED() == 1){
      estimate->setAlpha(0.0);
      estimate->setBetaOrSigmaM(pow(((targetValues->getVariance12() * (targetValues->getKurtosis12() - 3.0)) / 3.0),0.5));
      estimate->setSigmaE(0.0);
      estimate->setD1(3.0 / (targetValues->getKurtosis12() - 3.0));
      estimate->setD2(0.0);
      estimate->setD3(0.0);
      estimate->setD4(0.0);
    }

    // Extreme value distribution
    if(programParameters->getMED() == 2){
      estimate->setAlpha(0.0);
      estimate->setBetaOrSigmaM(pow(6.0 * ((targetValues->getVariance12() * (targetValues->getKurtosis12() - 3.0)) / (27.0 / 5.0)) * (pow(3.1415926535897932,-2.0)),0.5));
      estimate->setSigmaE(0.0);
      estimate->setD1(0.5 * ((27.0 / 5.0) / (targetValues->getKurtosis12() - 3.0)) * (1.0 + (targetValues->getSkewness12() / 1.13955) * pow((27.0 / 5.0) / (targetValues->getKurtosis12() - 3.0),0.5)));
      estimate->setD2(0.5 * ((27.0 / 5.0) / (targetValues->getKurtosis12() - 3.0)) * (1.0 - (targetValues->getSkewness12() / 1.13955) * pow((27.0 / 5.0) / (targetValues->getKurtosis12() - 3.0),0.5)));
      estimate->setD3(0.0);
      estimate->setD4(0.0);
    }

    // Gamma distribution
    if(programParameters->getMED() == 3){
      cout << "\nNo analytic parameter estimation for the M-gamma model available!" << endl;
    }

  } else {

    // Use of an outgroup

    // Normal distribution
    if(programParameters->getMED() == 1){
      estimate->setAlpha(0.0);
      estimate->setBetaOrSigmaM(pow(((targetValues->getVariance12() * (targetValues->getKurtosis12() - 3.0)) / 3.0),0.5));
      estimate->setSigmaE(0.0);
      estimate->setD1((3.0 * (targetValues->getVariance12() + targetValues->getVariance13() - targetValues->getVariance23())) / (2.0 * (targetValues->getKurtosis12() - 3.0) * targetValues->getVariance12()));
      estimate->setD2((3.0 * (targetValues->getVariance12() - targetValues->getVariance13() + targetValues->getVariance23())) / (2.0 * (targetValues->getKurtosis12() - 3.0) * targetValues->getVariance12()));
      estimate->setD3((3.0 * (-targetValues->getVariance12() + targetValues->getVariance13() + targetValues->getVariance23())) / (2.0 * (targetValues->getKurtosis12() - 3.0) * targetValues->getVariance12()));
      estimate->setD4(0.0);
    }

    // Extreme value distribution
    if(programParameters->getMED() == 2){
      estimate->setAlpha(0.0);
      estimate->setBetaOrSigmaM(pow(6.0 * ((targetValues->getVariance12() * (targetValues->getKurtosis12() - 3.0)) / (27.0 / 5.0)) * (pow(3.1415926535897932,-2.0)),0.5));
      estimate->setSigmaE(0.0);
      estimate->setD1(0.5 * (targetValues->getVariance12() + targetValues->getVariance13() - targetValues->getVariance23()) * pow((targetValues->getVariance12() * (targetValues->getKurtosis12() - 3.0)) / (27.0 / 5.0),-1.0));
      estimate->setD2(0.5 * (targetValues->getVariance12() - targetValues->getVariance13() + targetValues->getVariance23()) * pow((targetValues->getVariance12() * (targetValues->getKurtosis12() - 3.0)) / (27.0 / 5.0),-1.0));
      estimate->setD3(0.5 * ((27.0 / 5.0) / (targetValues->getKurtosis13() - 3.0)) * (1.0 - (targetValues->getSkewness13() / 1.13955) * pow((27.0 / 5.0) / (targetValues->getKurtosis13() - 3.0),0.5)));
      estimate->setD4(0.5 * ((27.0 / 5.0) / (targetValues->getKurtosis13() - 3.0)) * (1.0 + (targetValues->getSkewness13() / 1.13955) * pow((27.0 / 5.0) / (targetValues->getKurtosis13() - 3.0),0.5)) - estimate->getD1());
    }

    // Gamma distribution
    if(programParameters->getMED() == 3){
      cout << "\nNo analytic parameter estimation for the M-gamma model available!" << endl;
    }
  }

  // Quality has been reached per definition (No quality criterion exists)
  estimate->setObjectiveValue(0.0);
  estimate->setQualityValue(1.0);
}
