/***************************************************************************
                          controller.h  -  description
                             -------------------
    begin                : Thu Apr 29 2004
    copyright            : (C) 2004 by Michael Rosskopf
    email                : rosskopf@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  Description:
A Controller-object implements all use cases of the package and is initialized
directly from the Emogee-object.
*/

#ifndef CONTROLLER_H
#define CONTROLLER_H

#include <iostream.h>
#include <time.h>

#include "programparameters.h"
#include "mathfunctions.h"
#include "randomnumbergenerator.h"
#include "random.h"
#include "modelparameters.h"
#include "threetree.h"
#include "data.h"
#include "analyticestimator.h"
#include "optimizer.h"
#include "mloptimizer.h"
#include "resultcollector.h"
#include "graphictools.h"

/**
  *@author Michael Rosskopf
  */

class Controller {

public: 

  // Constructors and destructors
  Controller(char*,char*,double);
  ~Controller();

  // performDataGeneration: Implements data generation
  void performDataGeneration(char*);

  // performEstimationOnRealData: Implements parameter estimation on real data
  void performEstimationOnRealData(char*);

  // estimateFromTwoSamples:
  // Estimates model parameters for the expression difference between a pair of individuals
  // (called by performEstimationOnRealData in for analysisType == 2)
  void estimateFromTwoSamples(Data*,DistributionMeasures*,double*,double*,long,long);

  // performBootstrappingSeries:
  // Implements a bootstrapping and estimates model parameters on the bootstrap data
  void performBootstrappingSeries(char*);

  // performGenerateEstimateSeries:
  // Generates data and estimates model parameters on the generated data
  void performGenerateEstimateSeries(char*);
  
private:

  ProgramParameters* programParameters; // ProgramParameters specify what to do
  RandomNumberGenerator* rnd; // RandomNumberGenerator is used for data generation
  Random* gammarnd; // Random is used if gamma distributed random variables are used
  ThreeTree* threeTree; // ThreeTree implements data generation
  ModelParameters* modelParameters; // ModelParameters describes the parameters of the different models
  AnalyticEstimator* analyticEstimator; // Implements analytic parameter estimation for simple models
  Optimizer* optimizer; // Implements parameter estimation by optimisation methods
  MLOptimizer* mlOptimizer; // Implements parameter estimation by optimisation depending on maximum likelihood
  Data* data; // Stores a loaded data set
  ResultCollector* resultCollector; // stores the results

};

#endif
