/***************************************************************************
                          distributionmeasures.h  -  description
                             -------------------
    begin                : Mon Feb 16 2004
    copyright            : (C) 2004 by Michael Rosskopf
    email                : rosskopf@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  Description:
An DistributionMeasure-object contains the moments of a distribution, but it contains
also information necessary for parameter estimation of the M-gamma model.
*/

#ifndef DISTRIBUTIONMEASURES_H
#define DISTRIBUTIONMEASURES_H
#include <iostream.h>

#include "programparameters.h"

/**
  *@author Michael Rosskopf
  */

class DistributionMeasures {

public:

  // Constructors and destructors
  DistributionMeasures();
  ~DistributionMeasures();

  // get-and set-methods
  double getVarianceWithin1();
  double getVarianceWithin2();
  double getVarianceWithin3();
  double getVariance12();
  double getVariance13();
  double getVariance23();
  double getSkewness12();
  double getSkewness13();
  double getSkewness23();
  double getKurtosis12();
  double getKurtosis13();
  double getKurtosis23();
  double getS1iSkew();
  double getS2iSkew();
  double getS1iPearsonsSkew();
  double getS2iPearsonsSkew();
  long getOverallS1iCounter();
  long getOverallS2iCounter();
  int getSide();
  int getMirrored();
  void setVarianceWithin1(double);
  void setVarianceWithin2(double);
  void setVarianceWithin3(double);
  void setVariance12(double);
  void setVariance13(double);
  void setVariance23(double);
  void setSkewness12(double);
  void setSkewness13(double);
  void setSkewness23(double);
  void setKurtosis12(double);
  void setKurtosis13(double);
  void setKurtosis23(double);
  void setS1iSkew(double);
  void setS2iSkew(double);
  void setS1iPearsonsSkew(double);
  void setS2iPearsonsSkew(double);
  void setOverallS1iCounter(long);
  void setOverallS2iCounter(long);
  void setSide(int);
  void setMirrored(int);

  // toString
  void toString(ProgramParameters*);

  private:

  double varianceWithin1;
  double varianceWithin2;
  double varianceWithin3;
  double variance12;
  double variance13;
  double variance23;
  double skewness12;
  double skewness13;
  double skewness23;
  double kurtosis12;
  double kurtosis13;
  double kurtosis23;
  double s1iSkew;
  double s2iSkew;
  double s1iPearsonsSkew;
  double s2iPearsonsSkew;
  long overallS1iCounter;
  long overallS2iCounter;
  int side;      // 0 = Use s1iSkew to optimize
                 // 1 = Use s2iSkew to optimze
                 // 2 = Mixed (set after calculation) 
  int mirrored;  // 0 = Positive-skewed mutation effects
                 // 1 = Negative-skewed muattion effects
                 // 2 = Mixed (set after calculation)
};

#endif
