/***************************************************************************
                          graphictools.cpp  -  description
                             -------------------
    begin                : Wed Dec 1 2004
    copyright            : (C) 2004 by Michael Rosskopf
    email                : rosskopf@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "graphictools.h"

// Constructors and destructors
GraphicTools::GraphicTools(){
}
GraphicTools::~GraphicTools(){
}

// drawHistogram: Different methods to draw a histogram from data
//
// Draw a descretised distribution data[] with a given length
void GraphicTools::drawHistogram(double data[], long length){
  ofstream file("plot.txt");
  for(long i = 0; i < length; i++){
    file << i + 1 << "\t" << data[i] << endl;
    file << data[i] << endl;
  }
  system("gnuplot plotterscript");
  system("kghostview plot.ps");
}

// Discretises a distribution data[] with a given length to a numberOfBuckets
// and draws it
void GraphicTools::drawHistogram(double data[], long length, long numberOfBuckets){

  // Initialisation
  double greatest = -99999999;
  double smallest = 99999999;
  for(long i = 0; i < length; i++){
    if(data[i] < smallest) smallest = data[i];
    if(data[i] > greatest) greatest = data[i];
  }
  double range = greatest - smallest;
  double* buckets = new double[numberOfBuckets];                    // Array of buckets
  for(long i = 0; i < numberOfBuckets; i++){
    buckets[i] = 0;
  }

  long target;
  double buffer;
  // Filling the buckets
  if(range != 0){
    for(long i = 0; i < length; i++){
      buffer = data[i];
      buffer = buffer - smallest;
      target = long((buffer / range) * numberOfBuckets);
      if(target > (numberOfBuckets - 1)){
        target = (numberOfBuckets - 1);
      }
      buckets[target]++;
    }
  } else {
    buckets[long(numberOfBuckets / 2)] = length;
  }

  for(long i = 0; i < numberOfBuckets; i++){
    buckets[i] = buckets[i] / length;
  }

  drawHistogram(buckets,numberOfBuckets);

  delete buckets;
}

// Discretises a distribution data[] with a given length to a numberOfBuckets
// and draws it. Values which are out of the boundaries _smallest and _greatest
// are mapped to the first and the last bucket, respectively.
void GraphicTools::drawHistogram(double data[], long length, long numberOfBuckets, double _smallest, double _greatest){

  // Initialisation
  double greatest = _greatest;
  double smallest = _smallest;
  double range = greatest - smallest;
  double* buckets = new double[numberOfBuckets];                    // Array of buckets
  for(long i = 0; i < numberOfBuckets; i++){
    buckets[i] = 0;
  }

  long target;
  double buffer;
  // Filling the buckets
  if(range != 0){
    for(long i = 0; i < length; i++){
      buffer = data[i];
      buffer = buffer - smallest;
      target = long((buffer / range) * numberOfBuckets);
      if(target > (numberOfBuckets - 1)){
        target = (numberOfBuckets - 1);
      }
      if(target < 0) target = 0;
      buckets[target]++;
    }
  } else {
    buckets[long(numberOfBuckets / 2)] = length;
  }

  for(long i = 0; i < numberOfBuckets; i++){
    buckets[i] = buckets[i] / length;
  }

  ofstream file("plot.txt");
  for(long i = 0; i < numberOfBuckets; i++){
    file << smallest + ((double(i) / double(numberOfBuckets)) * range) << "\t" << buckets[i] << endl;
  }
  system("gnuplot plotterscript");
  system("kghostview plot.ps");

  delete buckets;
}

// Discretises a distribution data[] with a given length to a numberOfBuckets
// and draws it. Values which are out of the boundaries _smallest and _greatest
// are ignored, respectively.
void GraphicTools::drawHistogramWithTruncation(double data[], long length, long numberOfBuckets, double _smallest, double _greatest){

  // Initialisation
  double greatest = _greatest;
  double smallest = _smallest;
  double range = greatest - smallest;
  double* buckets = new double[numberOfBuckets];                    // Array of buckets
  for(long i = 0; i < numberOfBuckets; i++){
    buckets[i] = 0;
  }

  long target;
  double buffer;
  // Filling the buckets
  if(range != 0){
    for(long i = 0; i < length; i++){
      buffer = data[i];
      buffer = buffer - smallest;
      target = long((buffer / range) * numberOfBuckets);
      if((target >= 0) && (target < numberOfBuckets - 1)){
        buckets[target]++;
      }
    }
  } else {
    buckets[long(numberOfBuckets / 2)] = length;
  }

  for(long i = 0; i < numberOfBuckets; i++){
    buckets[i] = buckets[i] / length;
  }

  ofstream file("plot.txt");
  for(long i = 0; i < numberOfBuckets; i++){
    file << smallest + ((double(i) / double(numberOfBuckets)) * range) << "\t" << buckets[i] << endl;
  }
  system("gnuplot plotterscript");
  system("kghostview plot.ps");

  delete buckets;
}

// calcHistogramPlot: Different methods to bin data, but without visualisation

// This method discretises the distribution data[] with length length to a numberOfBuckets
// using given boundaries _smallest and _greatest. The results are stored in output[].
// Values outside the boundaries are mapped to the buckets closest to the bounds.
void GraphicTools::calcHistogramPlot(double data[], long length, double output[], long numberOfBuckets, double _smallest, double _greatest){

  // Initialisation
  double greatest = _greatest;
  double smallest = _smallest;
  double range = greatest - smallest;
  for(long i = 0; i < numberOfBuckets; i++){
    output[i] = 0.0;
  }

  long target;
  double buffer;
  // Filling the buckets
  if(range != 0){
    for(long i = 0; i < length; i++){
      buffer = data[i];
      buffer = buffer - smallest;
      target = long((buffer / range) * numberOfBuckets);
      if(target > (numberOfBuckets - 1)){
        target = (numberOfBuckets - 1);
      }
      if(target < 0) target = 0;
      output[target]++;
    }
  } else {
    output[long(numberOfBuckets / 2)] = length;
  }

  for(long i = 0; i < numberOfBuckets; i++){
    output[i] = output[i] / length;
  }
}

// This method discretises the distribution data[] with length length 
// to a numberOfBuckets and stores it in output[].
// The values of the pointers *smallest and *greatest
// which represent boundaries of the histogram are calculated.
void GraphicTools::calcHistogramPlot(double data[], long length, double output[], long numberOfBuckets, double *smallest, double *greatest){

  // Initialisation
  for(long i = 0; i < length; i++){
    if(data[i] < *smallest) *smallest = data[i];
    if(data[i] > *greatest) *greatest = data[i];
  }
  double range = *greatest - *smallest;
  for(long i = 0; i < numberOfBuckets; i++){
    output[i] = 0.0;
  }

  long target;
  double buffer;
  // Filling the buckets
  if(range != 0){
    for(long i = 0; i < length; i++){
      buffer = data[i];
      buffer = buffer - *smallest;
      target = long((buffer / range) * numberOfBuckets);
      if(target > (numberOfBuckets - 1)){
        target = (numberOfBuckets - 1);
      }
      if(target < 0) target = 0;
      output[target]++;
    }
  } else {
    output[long(numberOfBuckets / 2)] = length;
  }

  for(long i = 0; i < numberOfBuckets; i++){
    output[i] = output[i] / length;
  }
}

// This method discretises the distribution data[] with length length 
// to a numberOfBuckets and stores it in output[].
// Values outside the boundaries _smallest and _greatest, respectively, are ignored.
void GraphicTools::calcHistogramPlotWithTruncation(double data[], long length, double output[], long numberOfBuckets, double _smallest, double _greatest){

  // Initialisation
  double greatest = _greatest;
  double smallest = _smallest;
  double range = greatest - smallest;
  for(long i = 0; i < numberOfBuckets; i++){
    output[i] = 0.0;
  }

  long target;
  double buffer;
  // Filling the buckets
  if(range != 0){
    for(long i = 0; i < length; i++){
      buffer = data[i];
      buffer = buffer - smallest;
      target = long((buffer / range) * numberOfBuckets);
      if((target >= 0) && (target < numberOfBuckets - 1)){
        output[target]++;
      }
    }
  } else {
    output[long(numberOfBuckets / 2)] = length;
  }

  for(long i = 0; i < numberOfBuckets; i++){
    output[i] = output[i] / length;
  }
}
