/***************************************************************************
 *   Copyright (C) 2006 by Michael Rosskopf                                *
 *   rosskopf@cs.uni-duesseldorf.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "modelparameters.h"

// Constructors and destructors
ModelParameters::ModelParameters(){
  setNull();
  results = new DistributionMeasures();
}
ModelParameters::~ModelParameters(){
  delete results;
}

// Get- and set-methods
double ModelParameters::getAlpha(){
  return alpha;
}
double ModelParameters::getBetaOrSigmaM(){
  return betaOrSigmaM;
}
double ModelParameters::getSigmaE(){
  return sigmaE;
}
double ModelParameters::getD1(){
  return d1;
}
double ModelParameters::getD2(){
  return d2;
}
double ModelParameters::getD3(){
  return d3;
}
double ModelParameters::getD4(){
  return d4;
}
double ModelParameters::getObjectiveValue(){
  return objectiveValue;
}
// Enumeration starts with 0
double ModelParameters::getQualityValue(){
  return qualityValue;
}
DistributionMeasures* ModelParameters::getResults(){
  return results;
}

void ModelParameters::setAlpha(double _alpha){
  alpha = _alpha;
}
void ModelParameters::setBetaOrSigmaM(double _betaOrSigmaM){
  betaOrSigmaM = _betaOrSigmaM;
}
void ModelParameters::setSigmaE(double _sigmaE){
  sigmaE = _sigmaE;
}
void ModelParameters::setD1(double _d1){
  d1 = _d1;
}
void ModelParameters::setD2(double _d2){
  d2 = _d2;
}
void ModelParameters::setD3(double _d3){
  d3 = _d3;
}
void ModelParameters::setD4(double _d4){
  d4 = _d4;
}
void ModelParameters::setObjectiveValue(double _objectiveValue){
  objectiveValue = _objectiveValue;
}
void ModelParameters::setQualityValue(double _qualityValue){
  qualityValue = _qualityValue;
}
// setNull: Set all variables to 0
void ModelParameters::setNull(){
  alpha = 0.0;
  betaOrSigmaM = 0.0;
  sigmaE = 0.0;
  d1 = 0.0;
  d2 = 0.0;
  d3 = 0.0;
  d4 = 0.0;
  objectiveValue = 0.0;
  qualityValue = 0.0;
}

// toString: Output of the individual on the screen
void ModelParameters::toString(ProgramParameters* programParameters){
  long mED = programParameters->getMED();
  if(programParameters->getMED() == 1){
    cout << "\nsigma_m:    " << betaOrSigmaM << endl;
  }
  if(programParameters->getMED() == 2){
    cout << "\nbeta:       " << betaOrSigmaM << endl;
  }
  if(programParameters->getMED() == 3){
    cout << "\nalpha:      " << alpha << endl;
    cout << "beta:       " << betaOrSigmaM << endl;
  }
  if(programParameters->getNonMutationalEffects() == 2) cout << "sigma_e:    " << sigmaE << endl;
  if(programParameters->getMED() == 1 && programParameters->getSample3() == 0){
    cout << "d:          " << d1 << endl;
  } else {
    cout << "d_1:        " << d1 << endl;
    cout << "d_2:        " << d2 << endl;
  }
  if(programParameters->getSample3() != 0){
    cout << "d_3:        " << d3 << endl;
    if(programParameters->getMED() != 1){
      cout << "d_4:        " << d4 << endl;
      cout << "d_3+d_4:    " << d3 + d4 << endl;
    }
  }
  if(programParameters->getNonMutationalEffects() == 1){
    if(programParameters->getMED() == 3){
      cout << "\nDistance:   " << objectiveValue << endl;
      cout << "Quality:    " << qualityValue << endl;
    }
  } else {
    if(programParameters->getMethod() == 1) cout << "\nChi^2:      " << objectiveValue << endl;
    if(programParameters->getMethod() == 2) cout << "\n-log-like.: " << objectiveValue << endl;
    cout << "Quality:    " << qualityValue << endl;
  }
  if(programParameters->getMED() == 3){
    if(results->getSide() == 0) cout << "Use S1i-Genes    " << endl;
    if(results->getSide() == 1) cout << "Use S2i-Genes    " << endl;
    if(results->getSide() == 2) cout << "Use mean of both " << endl;
    if(results->getMirrored() == 0) cout << "Positive skewed mutation effects" << endl;
    if(results->getMirrored() == 1) cout << "Negative skewed mutation effects" << endl;
    if(results->getMirrored() == 2) cout << "Positive AND negative mutation effects" << endl;
  }
}

// addUpValues: Adds the passed model parameters to the existing values.
// Important for calculating the mean (in combination with divideValues).
void ModelParameters::addUpValues(ModelParameters* newEstimate){
  alpha = alpha + newEstimate->getAlpha();
  betaOrSigmaM = betaOrSigmaM + newEstimate->getBetaOrSigmaM();
  sigmaE = sigmaE + newEstimate->getSigmaE();
  d1 = d1 + newEstimate->getD1();
  d2 = d2 + newEstimate->getD2();
  d3 = d3 + newEstimate->getD3();
  d4 = d4 + newEstimate->getD4();
}

// divideValues: Divides the model parameters by the factor.
// Important for calculating the mean (in combination with addUpValues).
void ModelParameters::divideValues(double factor){
  alpha = alpha / factor;
  betaOrSigmaM = betaOrSigmaM / factor;
  sigmaE = sigmaE / factor;
  d1 = d1 / factor;
  d2 = d2 / factor;
  d3 = d3 / factor;
  d4 = d4 / factor;
}

// setParametersMGammaGivenAlpha: Sets the parameters given alpha (M-gamma model).
void ModelParameters::setParametersMGammaGivenAlpha(ProgramParameters *programParameters,DistributionMeasures *targetValues,double _alpha){
  alpha = _alpha;
  sigmaE = 0.0;
  if(programParameters->getMED() == 3){
    betaOrSigmaM = pow((targetValues->getVariance12() * (targetValues->getKurtosis12() - 3.0)) / (3.0 * alpha + 6.0),0.5);
    d1 = (3.0 * targetValues->getVariance12() + 1.5 * alpha * targetValues->getVariance12() + 3.0 * targetValues->getVariance13() + 1.5 * alpha * targetValues->getVariance13() - 3.0 * targetValues->getVariance23() - 1.5 * alpha * targetValues->getVariance23()) / (alpha * (targetValues->getKurtosis12() - 3.0) * targetValues->getVariance12());
    d2 = (3.0 * targetValues->getVariance12() + 1.5 * alpha * targetValues->getVariance12() - 3.0 * targetValues->getVariance13() - 1.5 * alpha * targetValues->getVariance13() + 3.0 * targetValues->getVariance23() + 1.5 * alpha * targetValues->getVariance23()) / (alpha * (targetValues->getKurtosis12() - 3.0) * targetValues->getVariance12());
    d3 = ((3.0 + 1.5 * alpha) * pow(targetValues->getKurtosis12() - 3.0,0.5) * pow(targetValues->getVariance12(),2.0) * targetValues->getVariance13() - 1.29903810567665 * pow(pow(2.0 + alpha,3.0) * pow(targetValues->getSkewness13(),2.0) * pow(targetValues->getVariance12(),3.0) * pow(targetValues->getVariance13(),3.0),0.5)) / (alpha * pow(targetValues->getKurtosis12() - 3.0,1.5) * pow(targetValues->getVariance12(),3.0));
    d4 = ((1.29903810567665 * pow(pow(2.0 + alpha,3.0) * pow(targetValues->getSkewness13(),2.0) * pow(targetValues->getVariance12(),3.0) * pow(targetValues->getVariance13(),3.0),0.5)) / (alpha * pow(targetValues->getKurtosis12() - 3.0,1.5) * pow(targetValues->getVariance12(),2.0)) + (  (-1.5 - 3.0 / alpha) * targetValues->getVariance12() + 1.5 * targetValues->getVariance23() + (3.0 * targetValues->getVariance23() / alpha)   ) / (targetValues->getKurtosis12() - 3.0)) / targetValues->getVariance12();
  }
}

// setParametersMENormalGivenSigmaE: Sets the parameters given sigma_e (M&E-normal model)
void ModelParameters::setParametersMENormalGivenSigmaE(ProgramParameters *programParameters,DistributionMeasures *targetValues,double error){
  sigmaE = error;
  betaOrSigmaM = targetValues->getVariance12() * pow((targetValues->getKurtosis12() - 3.0) / (3.0 * targetValues->getVariance12() - 6.0 * error * error),0.5);
  d1 = (3.0 * pow(targetValues->getVariance12(),2.0) - 12.0 * pow(error,2.0) * (targetValues->getVariance12() - pow(error,2.0))) / (pow(targetValues->getVariance12(),2.0) * targetValues->getKurtosis12() - 3.0 * pow(targetValues->getVariance12(),2.0));
  if(d1 < 0.0) d1 = d1 * (-1.0);
  d2 = 0.0;
  d3 = 0.0;
  d4 = 0.0;
}

// setParametersMEExtremeGivenSigmaE: Sets the parameters given sigma_e (M&E-extreme model).
void ModelParameters::setParametersMEExtremeGivenSigmaE(ProgramParameters *programParameters,DistributionMeasures *targetValues,double error){
  double buffer = 0.0;
  sigmaE = error;

  betaOrSigmaM = (pow(-30.0 + 10.0 * targetValues->getKurtosis12(),0.5) * targetValues->getVariance12()) / (3.1415926535897932 * pow(-18.0 * pow(sigmaE,2.0) + 9.0 * targetValues->getVariance12(),0.5));
  d1 = -(27.0 / (30.0 - 10.0 * targetValues->getKurtosis12()));
  buffer = -((403909000000000000000.0 * pow(targetValues->getSkewness12(),2.0)) / (pow(30.0 - 10.0 * targetValues->getKurtosis12(),3.0)));
  buffer = buffer + ((3231270000000000000000.0 * pow(sigmaE,6.0) * pow(targetValues->getSkewness12(),2.0)) / (pow(30.0 - 10.0 * targetValues->getKurtosis12(),3.0) * pow(targetValues->getVariance12(),3.0)));
  buffer = buffer - ((4846900000000000000000.0 * pow(sigmaE,4.0) * pow(targetValues->getSkewness12(),2.0)) / (pow(30.0 - 10.0 * targetValues->getKurtosis12(),3.0) * pow(targetValues->getVariance12(),2.0)));
  buffer = buffer + ((2423450000000000000000.0 * pow(sigmaE,2.0) * pow(targetValues->getSkewness12(),2.0)) / (pow(30.0 - 10.0 * targetValues->getKurtosis12(),3.0) * targetValues->getVariance12()));

  buffer = pow(buffer,0.5);
  d1 = d1 + 0.00000000866335 * buffer;
  d1 = d1 - ((108.0 * pow(sigmaE,4.0)) / ((30.0 - 10.0 * targetValues->getKurtosis12()) * pow(targetValues->getVariance12(),2.0)));
  d1 = d1 + ((108.0 * pow(sigmaE,2.0)) / ((30.0 - 10.0 * targetValues->getKurtosis12()) * targetValues->getVariance12()));

  d2 = -buffer - ((12466300000.0 * pow(sigmaE,4.0) - 12466300000.0 * pow(sigmaE,2.0) * targetValues->getVariance12() + 3116580000.0 * pow(targetValues->getVariance12(),2.0)) / ((30.0 - 10.0 * targetValues->getKurtosis12()) * pow(targetValues->getVariance12(),2.0)));
  d2 = d2 * 0.00000000866335;

  // Since skewness is always squared, rates are flipped in case of negative skewness
  if(targetValues->getSkewness12() < 0.0){
    buffer = d1;
    d1 = d2;
    d2 = buffer;
  }

  d3 = 0.0;
  d4 = 0.0;
}

// calcMEExtremeMaximalSigmaE: Calculates the maximal sigma_e (M&E-extreme model).
// Maximal sigmaE is necessary to avoid nan-problems
double ModelParameters::calcMEExtremeMaximalSigmaE(ProgramParameters *programParameters,DistributionMeasures *targetValues){
  double maximalSigmaE;
  maximalSigmaE = pow(0.5 * targetValues->getVariance12(),0.5);
  return maximalSigmaE;
}

// Methods for loading and saving model parameters

bool ModelParameters::loadModelParameters(const char* filename){

  // Check whether the file exist, return if not
  if(!checkFile(filename)){
    cerr << "Can not open model file " << filename << "!" << endl;
    return false;
  }

  char charbuffer;
  char stringbuffer[64];
  ifstream file(filename);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  alpha = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  betaOrSigmaM = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  sigmaE = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  d1 = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  d2 = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  d3 = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  d4 = atof(stringbuffer);

  file.close();
  return true;
}

// checkFile: Checks if a file exists or not (returns true or false)
bool ModelParameters::checkFile(const char* filename)  {
  ifstream file;
  file.open(filename);
  if(file){
    file.close();
    return true;
  }
  file.close();
  return false;
}

bool ModelParameters::saveModelParameters(const char* filename){

  // Check whether the file exist, return if not
  if(checkFile(filename)){
    cerr << filename << " still exists! Overwrite (y/n) ? ";
    char step;
    cin >> step;
    if(step != 'y') return false;
  }

  ofstream file(filename);
  file << "Parameters\n";
  file << "alpha:           " << alpha << "\n";
  file << "beta or sigma_m: " << betaOrSigmaM << "\n";
  file << "sigma_e:         " << sigmaE << "\n";
  file << "d_1:             " << d1 << "\n";
  file << "d_2:             " << d2 << "\n";
  file << "d_3:             " << d3 << "\n";
  file << "d_4:             " << d4 << "\n";
  file.close();
  return true;
}
