/***************************************************************************
 *   Copyright (C) 2006 by Michael Rosskopf                                *
 *   rosskopf@cs.uni-duesseldorf.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*  Description:
This class describes the model parameters of the different models.
*/

#ifndef MODELPARAMETERS_H
#define MODELPARAMETERS_H

#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include "programparameters.h"
#include "randomnumbergenerator.h"
#include "distributionmeasures.h"

/**
  *@author Michael Rosskopf
  */

class ModelParameters{

  public:
 
  // Constructors and destructors
  ModelParameters();
  ~ModelParameters();

  // get- and set-methods
  double getAlpha();
  double getBetaOrSigmaM();
  double getSigmaE();
  double getD1();
  double getD2();
  double getD3();
  double getD4();
  double getObjectiveValue();
  double getQualityValue();
  DistributionMeasures* getResults();
  void setAlpha(double);
  void setBetaOrSigmaM(double);
  void setSigmaE(double);
  void setD1(double);
  void setD2(double);
  void setD3(double);
  void setD4(double);
  void setObjectiveValue(double);
  void setQualityValue(double);
  void setNull();

  // toString
  void toString(ProgramParameters*);

  // addUpValues: Adds the passed model parameters to the existing values.
  void addUpValues(ModelParameters*);

  // divideValues: Divides the model parameters by the factor.
  void divideValues(double);

  // setParametersMGammaGivenAlpha: Sets the parameters given alpha (M-gamma model)
  void setParametersMGammaGivenAlpha(ProgramParameters*,DistributionMeasures*,double);

  // setParametersMENormalGivenSigmaE: Sets the parameters given sigma_e (M&E-normal model)
  void setParametersMENormalGivenSigmaE(ProgramParameters*,DistributionMeasures*,double);

  // setParametersMEExtremeGivenSigmaE: Sets the parameters given sigma_e (M&E-extreme model)
  void setParametersMEExtremeGivenSigmaE(ProgramParameters*,DistributionMeasures*,double);

  // calcMEExtremeMaximalSigmaE: Calculates the maximal sigma_e (M&E-extreme model)
  double calcMEExtremeMaximalSigmaE(ProgramParameters*,DistributionMeasures*);

  // Methods for loading and saving model parameters
  bool loadModelParameters(const char*);
  bool checkFile(const char*);
  bool saveModelParameters(const char*);

  private:

  double alpha;                   // Only used by the M-gamma model
  double betaOrSigmaM;            // beta if the M-gamma model is used
                                  // sigma_m if an M&E model is used
  double sigmaE;                  // Variance of the normal distributed error
  double d1;                      // Important: d if the M&E-normal model is used (d_1 otherwise)
  double d2;
  double d3;
  double d4;
  double objectiveValue;          // Objective function value 
                                  //  (meaning depends on the optimisation method)
  double qualityValue;            // Quality of a solution
  DistributionMeasures* results;  // Results of the last check of an individual
};

#endif
