/***************************************************************************
 *   Copyright (C) 2007 by Michael Rosskopf                                *
 *   rosskopf@cs.uni-duesseldorf.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should haverage received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "resultcollector.h"

// Constructors and destructors
ResultCollector::ResultCollector(){
  size = 0;
  modelParameterArray = 0;
}
ResultCollector::~ResultCollector(){
  for(long i = 0; i < size; i++){
    modelParameterArray[i].~ModelParameters();
    targetValues[i].~DistributionMeasures();
  }
//  delete modelParameterArray;
}

// init: Initializes the result collector with a specific size. 
//       The size describes the number results which can be stored.
void ResultCollector::init(long _size){
  // Initialization
  size = _size;
  modelParameterArray = new ModelParameters[size];
  targetValues = new DistributionMeasures[size];
}

// getSize: Returns the size.
long ResultCollector::getSize(){
  return size;
}

// getModelParameters: Returns the i-th stored result.
ModelParameters* ResultCollector::getModelParameters(long i){
  if(i< 0 || size < i){
    cerr << "ERROR in ResultCollector::getModelParameters(): Index " << i << " out of bounds!" << endl;
    char step;
    cin >> step;
    return 0;
  }
  return &modelParameterArray[i];
}

// setModelParameters: Stores a result at position i.
void ResultCollector::setModelParameters(long i,ModelParameters& modelParameters,DistributionMeasures& _targetValues){
  if(i < 0 || size < i){
    cerr << "ERROR in ResultCollector::setModelParameters(): Index " << i << " out of bounds!" << endl;
    char step;
    cin >> step;
    return;
  }
  modelParameterArray[i].setAlpha(modelParameters.getAlpha());
  modelParameterArray[i].setBetaOrSigmaM(modelParameters.getBetaOrSigmaM());
  modelParameterArray[i].setSigmaE(modelParameters.getSigmaE());
  modelParameterArray[i].setD1(modelParameters.getD1());
  modelParameterArray[i].setD2(modelParameters.getD2());
  modelParameterArray[i].setD3(modelParameters.getD3());
  modelParameterArray[i].setD4(modelParameters.getD4());
  modelParameterArray[i].setObjectiveValue(modelParameters.getObjectiveValue());
  modelParameterArray[i].setQualityValue(modelParameters.getQualityValue());

  modelParameterArray[i].getResults()->setVarianceWithin1(modelParameters.getResults()->getVarianceWithin1());
  modelParameterArray[i].getResults()->setVarianceWithin2(modelParameters.getResults()->getVarianceWithin2());
  modelParameterArray[i].getResults()->setVarianceWithin3(modelParameters.getResults()->getVarianceWithin3());
  modelParameterArray[i].getResults()->setVariance12(modelParameters.getResults()->getVariance12());
  modelParameterArray[i].getResults()->setVariance13(modelParameters.getResults()->getVariance13());
  modelParameterArray[i].getResults()->setVariance23(modelParameters.getResults()->getVariance23());
  modelParameterArray[i].getResults()->setSkewness12(modelParameters.getResults()->getSkewness12());
  modelParameterArray[i].getResults()->setSkewness13(modelParameters.getResults()->getSkewness13());
  modelParameterArray[i].getResults()->setSkewness23(modelParameters.getResults()->getSkewness23());
  modelParameterArray[i].getResults()->setKurtosis12(modelParameters.getResults()->getKurtosis12());
  modelParameterArray[i].getResults()->setKurtosis13(modelParameters.getResults()->getKurtosis13());
  modelParameterArray[i].getResults()->setKurtosis23(modelParameters.getResults()->getKurtosis23());
  modelParameterArray[i].getResults()->setS1iSkew(modelParameters.getResults()->getS1iSkew());
  modelParameterArray[i].getResults()->setS1iSkew(modelParameters.getResults()->getS1iSkew());
  modelParameterArray[i].getResults()->setS2iPearsonsSkew(modelParameters.getResults()->getS2iPearsonsSkew());
  modelParameterArray[i].getResults()->setS2iPearsonsSkew(modelParameters.getResults()->getS2iPearsonsSkew());
  modelParameterArray[i].getResults()->setSide(modelParameters.getResults()->getSide());
  modelParameterArray[i].getResults()->setMirrored(modelParameters.getResults()->getMirrored());

  targetValues[i].setVarianceWithin1(_targetValues.getVarianceWithin1());
  targetValues[i].setVarianceWithin2(_targetValues.getVarianceWithin2());
  targetValues[i].setVarianceWithin3(_targetValues.getVarianceWithin3());
  targetValues[i].setVariance12(_targetValues.getVariance12());
  targetValues[i].setVariance13(_targetValues.getVariance13());
  targetValues[i].setVariance23(_targetValues.getVariance23());
  targetValues[i].setSkewness12(_targetValues.getSkewness12());
  targetValues[i].setSkewness13(_targetValues.getSkewness13());
  targetValues[i].setSkewness23(_targetValues.getSkewness23());
  targetValues[i].setKurtosis12(_targetValues.getKurtosis12());
  targetValues[i].setKurtosis13(_targetValues.getKurtosis13());
  targetValues[i].setKurtosis23(_targetValues.getKurtosis23());
  targetValues[i].setS1iSkew(_targetValues.getS1iSkew());
  targetValues[i].setS2iSkew(_targetValues.getS2iSkew());
  targetValues[i].setS1iPearsonsSkew(_targetValues.getS1iPearsonsSkew());
  targetValues[i].setS2iPearsonsSkew(_targetValues.getS2iPearsonsSkew());
  targetValues[i].setSide(_targetValues.getSide());
  targetValues[i].setMirrored(_targetValues.getMirrored());
}

// checkFile: Checks if a file exists or not (returns true or false)
bool ResultCollector::checkFile(const char* filename)  {
  ifstream file;
  file.open(filename);
  if(file){
    file.close();
    return true;
  }
  file.close();
  return false;
}

// saverageCompleteParameterTable: Saverages all results in a table.
bool ResultCollector::saveCompleteParameterTable(const char* filename, bool ask){

  // Check whether the file exist, return if not
  if(ask && checkFile(filename)){
    cerr << filename << " still exists! Overwrite (y/n) ? ";
    char step;
    cin >> step;
    if(step != 'y') return false;
  }

  ofstream file(filename);
  for(long j = 0; j < size; j++){
    file << j + 1 << "\t";
    file << modelParameterArray[j].getAlpha() << "\t";
    file << modelParameterArray[j].getBetaOrSigmaM() << "\t";
    file << modelParameterArray[j].getSigmaE() << "\t";
    file << modelParameterArray[j].getD1() << "\t";
    file << modelParameterArray[j].getD2() << "\t";
    file << modelParameterArray[j].getD3() << "\t";
    file << modelParameterArray[j].getD4() << "\t";
    file << modelParameterArray[j].getD3() + modelParameterArray[j].getD4() << "\t";
    file << "***" << "\t";
    file << modelParameterArray[j].getObjectiveValue() << "\t";
//    file << modelParameterArray[j].getQualityValue() << "\t";
    if(modelParameterArray[j].getQualityValue() == 1.0){
      file << "Valid\t";
    } else {
      file << "Not valid! Limit reached!\t";
    }
    file << "***" << "\t";
    file << targetValues[j].getVarianceWithin1() << "\t";
    file << targetValues[j].getVarianceWithin2() << "\t";
    file << targetValues[j].getVarianceWithin3() << "\t";
    file << targetValues[j].getVariance12() << "\t";
    file << targetValues[j].getVariance13() << "\t";
    file << targetValues[j].getVariance23() << "\t";
    file << targetValues[j].getSkewness12() << "\t";
    file << targetValues[j].getSkewness13() << "\t";
    file << targetValues[j].getSkewness23() << "\t";
    file << targetValues[j].getKurtosis12() << "\t";
    file << targetValues[j].getKurtosis13() << "\t";
    file << targetValues[j].getKurtosis23() << "\t";
    file << targetValues[j].getS1iPearsonsSkew() << "\t";
    file << targetValues[j].getS2iPearsonsSkew() << "\n";
  }
  file.close();
  return true;
}

// saverageModelParameters: Saverages one result at position i.
bool ResultCollector::saveModelParameters(long i,const char* filename){

  if(i < 0 || i >= size) return false;

  ofstream file(filename,ios::app);

  file << i + 1 << "\t";
  file << modelParameterArray[i].getAlpha() << "\t";
  file << modelParameterArray[i].getBetaOrSigmaM() << "\t";
  file << modelParameterArray[i].getSigmaE() << "\t";
  file << modelParameterArray[i].getD1() << "\t";
  file << modelParameterArray[i].getD2() << "\t";
  file << modelParameterArray[i].getD3() << "\t";
  file << modelParameterArray[i].getD4() << "\t";
  file << modelParameterArray[i].getD3() + modelParameterArray[i].getD4() << "\t";
  file << "*" << "\t";
  file << modelParameterArray[i].getObjectiveValue() << "\t";
  if(modelParameterArray[i].getQualityValue() == 1.0){
    file << "Valid\t";
  } else {
    file << "Not valid! Limit reached!\t";
  }
  file << "*" << "\t";
  file << targetValues[i].getVarianceWithin1() << "\t";
  file << targetValues[i].getVarianceWithin2() << "\t";
  file << targetValues[i].getVarianceWithin3() << "\t";
  file << targetValues[i].getVariance12() << "\t";
  file << targetValues[i].getVariance13() << "\t";
  file << targetValues[i].getVariance23() << "\t";
  file << targetValues[i].getSkewness12() << "\t";
  file << targetValues[i].getSkewness13() << "\t";
  file << targetValues[i].getSkewness23() << "\t";
  file << targetValues[i].getKurtosis12() << "\t";
  file << targetValues[i].getKurtosis13() << "\t";
  file << targetValues[i].getKurtosis23() << "\t";
  file << targetValues[i].getS1iPearsonsSkew() << "\t";
  file << targetValues[i].getS2iPearsonsSkew() << "\n";
  file.close();
  return true;
}
