/***************************************************************************
 *   Copyright (C) 2007 by Michael Rosskopf                                *
 *   rosskopf@cs.uni-duesseldorf.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*  Description:
This class implements a container for parameter estimates of the M and M&E models
estimated with the optimizer or ml optimizer. The result of one estimation consists
of - depending on the used model - estimates for each parameter of the corresponding
model.
*/

#ifndef RESULTCOLLECTOR_H
#define RESULTCOLLECTOR_H

#include "modelparameters.h"
#include "distributionmeasures.h"

/**
  *@author Michael Rosskopf
  */

class ResultCollector{

public: 

  // Constructors and destructors
  ResultCollector();
  ~ResultCollector();

  // init: Initializes the result collector with a specific size. 
  //       The size describes the number results which can be stored.
  void init(long);

  // getSize: Returns the size.
  long getSize();

  // getModelParameters: Returns the i-th stored result.
  ModelParameters* getModelParameters(long);

  // setModelParameters: Stores a result at position i.
  void setModelParameters(long,ModelParameters&,DistributionMeasures&);

  // checkFile: Help routine which is necessary for the save-methods.
  bool checkFile(const char*);

  // saveCompleteParameterTable: Saves all results in a table.
  bool saveCompleteParameterTable(const char*,bool);

  // saveModelParameters: Saves one result at position i.
  bool saveModelParameters(long,const char*);

private:

  long size;                          // Number of model parameter estimates 
  ModelParameters* modelParameterArray;   // Array of model parameters
  DistributionMeasures* targetValues; // Array of the moments of the corresponding data

};

#endif
