/***************************************************************************
                          threetree.cpp  -  description
                             -------------------
    begin                : Fri Dec 17 2004
    copyright            : (C) 2004 by Michael Rosskopf
    email                : rosskopf@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "threetree.h"

// Constructors and destructors
ThreeTree::ThreeTree(){
}
ThreeTree::~ThreeTree(){
}

// simulationMGamma: Simulates data according to the M-gamma model
// in order to estimate the s1i- and s2i-skewness
void ThreeTree::simulationMGamma(ProgramParameters *programParameters,Data *data,long iterations,ModelParameters *modelParameters,DistributionMeasures *distributionMeasures,RandomNumberGenerator *rnd,Random *gammarnd){

  // Initialization
  alpha = modelParameters->getAlpha();
  betaOrSigmaM = modelParameters->getBetaOrSigmaM();
  sigmaE = modelParameters->getSigmaE();
  d1 = modelParameters->getD1();
  d2 = modelParameters->getD2();
  d3 = modelParameters->getD3();
  d4 = modelParameters->getD4();

  l_geneCounter = 0;
  r_geneCounter = 0;
  l_sum = 0;
  l_squaredSum = 0;
  l_cubedSum = 0;
  sliVar = 0;
  r_sum = 0;
  r_squaredSum = 0;
  r_cubedSum = 0;
  sriVar = 0;

  loopCounter = 0;
  n = iterations;

  MathFunctions mathFunctions = MathFunctions();
  double median;
  double* sliDistribution = new double[n];
  double* sriDistribution = new double[n];

  double* firstGroupSamples = new double[data->getSampleSize(programParameters->getSample1() - 1)];
  double* secondGroupSamples = new double[data->getSampleSize(programParameters->getSample2() - 1)];
  double* outGroupSamples = new double[data->getSampleSize(programParameters->getSample3() - 1)];

  if(programParameters->getSample3() == 0){

    cout << "Error: Chosen mutation effect distribution not supported!" << endl;

  } else {

    // With outgroup

    // Normal model
    if(programParameters->getMED() == 1){
      cout << "Error: Chosen mutation effect distribution not supported!" << endl;
    }

    // Extreme value model
    if(programParameters->getMED() == 2){
      cout << "Error: Chosen mutation effect distribution not supported!" << endl;
    }

    // Gamma model
    if(programParameters->getMED() == 3){
      distributionMeasures->setVariance12((alpha * pow(betaOrSigmaM,2.0) * (d1 + d2)) + 2.0 * pow(sigmaE,2.0));
      distributionMeasures->setVariance13((alpha * pow(betaOrSigmaM,2.0) * (d1 + d4 + d3)) + 2.0 * pow(sigmaE,2.0));
      distributionMeasures->setVariance23((alpha * pow(betaOrSigmaM,2.0) * (d2 + d4 + d3)) + 2.0 * pow(sigmaE,2.0));
      distributionMeasures->setSkewness12((2.0 * (d1 - d2)) / (pow(alpha,0.5) * pow(d1 + d2 + (2.0 * pow(sigmaE,2.0) / (alpha * pow(betaOrSigmaM,2.0))),1.5)));
      distributionMeasures->setSkewness13((2.0 * ((d1 + d4) - d3)) / (pow(alpha,0.5) * pow((d1 + d4) + d3 + (2.0 * pow(sigmaE,2.0) / (alpha * pow(betaOrSigmaM,2.0))),1.5)));
      distributionMeasures->setSkewness23((2.0 * ((d2 + d4) - d3)) / (pow(alpha,0.5) * pow((d2 + d4) + d3 + (2.0 * pow(sigmaE,2.0) / (alpha * pow(betaOrSigmaM,2.0))),1.5)));
      distributionMeasures->setKurtosis12(((3.0 * alpha + 6.0) * pow(alpha,2.0) * pow(betaOrSigmaM,4.0) * (d1 + d2)) / (alpha * pow(distributionMeasures->getVariance12(),2.0)) + 3.0);
      distributionMeasures->setKurtosis13(((3.0 * alpha + 6.0) * pow(alpha,2.0) * pow(betaOrSigmaM,4.0) * (d1 + d4 + d3)) / (alpha * pow(distributionMeasures->getVariance12(),2.0)) + 3.0);
      distributionMeasures->setKurtosis23(((3.0 * alpha + 6.0) * pow(alpha,2.0) * pow(betaOrSigmaM,4.0) * (d2 + d4 + d3)) / (alpha * pow(distributionMeasures->getVariance12(),2.0)) + 3.0);
      // Correction for simulation, if path over root has a negative edge
      if(d3 < 0 || d4 < 0){
        d3 = d3 + d4;
        d4 = 0;
      }

      do{
        // Evolution
        root = 0.0;

        ancestor = root;
        numberOfMutations = rnd->getPoissonDistributedNumber(d4);
        for(i = 0; i < numberOfMutations; i++){
          if(distributionMeasures->getMirrored() == 0){
            ancestor = ancestor + (gammarnd->rgamma(alpha,betaOrSigmaM) - (alpha * betaOrSigmaM));
          } else {
            ancestor = ancestor + (-gammarnd->rgamma(alpha,betaOrSigmaM) + (alpha * betaOrSigmaM));
          }
        }

        for(j = 0; j < data->getSampleSize(programParameters->getSample3() - 1); j++){
          outGroupSamples[j] = root;
          numberOfMutations = rnd->getPoissonDistributedNumber(d3);
          for(i = 0; i < numberOfMutations; i++){
            if(distributionMeasures->getMirrored() == 0){
              outGroupSamples[j] = outGroupSamples[j] + (gammarnd->rgamma(alpha,betaOrSigmaM) - (alpha * betaOrSigmaM));
            } else {
              outGroupSamples[j] = outGroupSamples[j] + (-gammarnd->rgamma(alpha,betaOrSigmaM) + (alpha * betaOrSigmaM));
            }
          }
        }

        for(j = 0; j < data->getSampleSize(programParameters->getSample1() - 1); j++){
          firstGroupSamples[j] = ancestor;
          numberOfMutations = rnd->getPoissonDistributedNumber(d1);
          for(i = 0; i < numberOfMutations; i++){
            if(distributionMeasures->getMirrored() == 0){
              firstGroupSamples[j] = firstGroupSamples[j] + (gammarnd->rgamma(alpha,betaOrSigmaM) - (alpha * betaOrSigmaM));
            } else {
              firstGroupSamples[j] = firstGroupSamples[j] + (-gammarnd->rgamma(alpha,betaOrSigmaM) + (alpha * betaOrSigmaM));
            }
          }
        }

        for(j = 0; j < data->getSampleSize(programParameters->getSample2() - 1); j++){
          secondGroupSamples[j] = ancestor;
          numberOfMutations = rnd->getPoissonDistributedNumber(d2);
          for(i = 0; i < numberOfMutations; i++){
            if(distributionMeasures->getMirrored() == 0){
              secondGroupSamples[j] = secondGroupSamples[j] + (gammarnd->rgamma(alpha,betaOrSigmaM) - (alpha * betaOrSigmaM));
            } else {
              secondGroupSamples[j] = secondGroupSamples[j] + (-gammarnd->rgamma(alpha,betaOrSigmaM) + (alpha * betaOrSigmaM));
            }
          }
        }

        for(i = 0; i < data->getSampleSize(programParameters->getSample1() - 1); i++){
          for(j = 0; j < data->getSampleSize(programParameters->getSample2() - 1); j++){
            for(k = 0; k < data->getSampleSize(programParameters->getSample3() - 1); k++){
              leftLeaf = firstGroupSamples[i];
              rightLeaf = secondGroupSamples[j];
              outLeaf = outGroupSamples[k];

              loopCounter++;

              if(((rightLeaf < leftLeaf) && (leftLeaf < outLeaf) || (outLeaf < leftLeaf) && (leftLeaf < rightLeaf))   ||   ((leftLeaf == outLeaf) && (rightLeaf != leftLeaf))){
                if(l_geneCounter < n){
                  difference = leftLeaf - rightLeaf;
                  l_sum = l_sum + difference;
                  l_squaredSum = l_squaredSum + (difference * difference);
                  l_cubedSum = l_cubedSum + (difference * difference * difference);
                  sliDistribution[l_geneCounter] = difference;
                  l_geneCounter++;
                }
              }
              if(((leftLeaf < rightLeaf) && (rightLeaf < outLeaf) || (outLeaf < rightLeaf) && (rightLeaf < leftLeaf))   ||   ((rightLeaf == outLeaf) && (leftLeaf != rightLeaf))){
                if(r_geneCounter < n){
                  difference = leftLeaf - rightLeaf;
                  r_sum = r_sum + difference;
                  r_squaredSum = r_squaredSum + (difference * difference);
                  r_cubedSum = r_cubedSum + (difference * difference * difference);
                  sriDistribution[r_geneCounter] = difference;
                  r_geneCounter++;
                }
              }
            }
          }
        }

      } while(((l_geneCounter < n) || (r_geneCounter < n)) && (loopCounter < (n * 10.0)));
      sliVar = (l_squaredSum - (l_sum * l_sum / n)) / n;
      distributionMeasures->setS1iSkew((1.0 / (n * pow(sliVar,1.5)) * (l_cubedSum - (3.0 * (l_sum / n) * l_squaredSum) + (3.0 * pow((l_sum / n),2.0) * l_sum) - (pow(l_sum,3.0) / pow(n,2.0)))));
      if(programParameters->getSkewType() == 2){
        mathFunctions.calcMedianRearrange(sliDistribution,n,&median);
        distributionMeasures->setS1iPearsonsSkew((3.0 * ((l_sum / n) - median)) / pow(sliVar,0.5));
      }
      sriVar = (r_squaredSum - (r_sum * r_sum / n)) / n;
      distributionMeasures->setS2iSkew((1.0 / (n * pow(sriVar,1.5)) * (r_cubedSum - (3.0 * (r_sum / n) * r_squaredSum) + (3.0 * pow((r_sum / n),2.0) * r_sum) - (pow(r_sum,3.0) / pow(n,2.0)))));
      if(programParameters->getSkewType() == 2){
        mathFunctions.calcMedianRearrange(sriDistribution,n,&median);
        distributionMeasures->setS2iPearsonsSkew((3.0 * ((r_sum / n) - median)) / pow(sriVar,0.5));
      }
    }
  }

  delete firstGroupSamples;
  delete secondGroupSamples;
  delete outGroupSamples;
  delete sliDistribution;
  delete sriDistribution;
}

// simulationME: Simulates data according to the M&E model
// in order to calculate the Chi^2-distance to analysed real data
void ThreeTree::simulationME(ProgramParameters *programParameters,Data *data,double differenceVector[],long iterations,ModelParameters *modelParameters,DistributionMeasures *distributionMeasures,RandomNumberGenerator *rnd,Random *gammarnd){

  // Initialization
  alpha = modelParameters->getAlpha();
  betaOrSigmaM = modelParameters->getBetaOrSigmaM();
  sigmaE = modelParameters->getSigmaE();
  d1 = modelParameters->getD1();
  d2 = modelParameters->getD2();
  d3 = modelParameters->getD3();
  d4 = modelParameters->getD4();

  loopCounter = 0;
  n = iterations;

  long i,j;

  double* firstGroupSamples = new double[data->getSampleSize(programParameters->getSample1() - 1)];
  double* secondGroupSamples = new double[data->getSampleSize(programParameters->getSample2() - 1)];
  double* outGroupSamples = new double[data->getSampleSize(programParameters->getSample3() - 1)];

  // Normal model
  if(programParameters->getMED() == 1){
    distributionMeasures->setVariance12(pow(betaOrSigmaM,2.0) * d1 + 2.0 * pow(sigmaE,2.0));
    distributionMeasures->setSkewness12(0.0);
    distributionMeasures->setKurtosis12((3.0 * pow(betaOrSigmaM,4.0) * pow(d1,2.0) + 3.0 * pow(betaOrSigmaM,4.0) * d1 + 12.0 * pow(sigmaE,4.0) + 12.0 * pow(betaOrSigmaM,2.0) * pow(sigmaE,2.0) * d1) / pow(pow(betaOrSigmaM,2.0) * d1 + 2.0 * pow(sigmaE,2.0),2.0));

    loopCounter = 0;

    do{
      for(i = 0; i < data->getSampleSize(programParameters->getSample1() - 1); i++){
        // Simulate environmental effect for 1st group
        firstGroupSamples[i] = rnd->getNormalDistributedNumber(0.0,sigmaE);
      }
      for(j = 0; j < data->getSampleSize(programParameters->getSample2() - 1); j++){
        secondGroupSamples[j] = 0.0;
        // Simulate mutation events
        numberOfMutations = rnd->getPoissonDistributedNumber(d1);
        for(i = 0; i < numberOfMutations; i++){
          secondGroupSamples[j] = secondGroupSamples[j] + (rnd->getNormalDistributedNumber(0.0,betaOrSigmaM));
        }
        // Simulate environmental effect for 2nd group
        secondGroupSamples[j] = secondGroupSamples[j] + rnd->getNormalDistributedNumber(0.0,sigmaE);
      }
      // Compare samples
      for(i = 0; i < data->getSampleSize(programParameters->getSample1() - 1); i++){
        for(j = 0; j < data->getSampleSize(programParameters->getSample2() - 1); j++){
          if(loopCounter < n) differenceVector[loopCounter] = firstGroupSamples[i] - secondGroupSamples[j];
          loopCounter++;
        }
      }
    } while(loopCounter < n);
  }

  // Extreme value model
  if(programParameters->getMED() == 2){
    distributionMeasures->setVariance12((pow(3.1415926535897932,2.0) * pow(betaOrSigmaM,2.0) / 6.0) * (d1 + d2) + 2.0 * pow(sigmaE,2.0));
    distributionMeasures->setSkewness12(1.13955 * ((d1 - d2) / pow(d1 + d2 + (2.0 * pow(sigmaE,2.0) / (pow(3.1415926535897932,2.0) * pow(betaOrSigmaM,2.0) / 6.0)),1.5)));
    distributionMeasures->setKurtosis12(3.0 + ((5.4 * (pow(3.1415926535897932,4.0) / 36.0) * pow(betaOrSigmaM,4.0) * (d1 + d2)) / pow(distributionMeasures->getVariance12(),2.0)));

    loopCounter = 0;

    // Correction for simulation, if path over root has a negative edge
    if(d1 < 0.0 && d2 > 0.0){
      d2 = d2 + d1;
      d1 = 0.0;
    }
    if(d2 < 0.0 && d1 > 0.0){
      d1 = d1 + d2;
      d2 = 0.0;
    }
    do{
      for(i = 0; i < data->getSampleSize(programParameters->getSample1() - 1); i++){
        // Simulate environmental effect for 1st group
        firstGroupSamples[i] = 0.0;
        numberOfMutations = rnd->getPoissonDistributedNumber(d1);
        for(j = 0; j < numberOfMutations; j++){
          firstGroupSamples[i] = firstGroupSamples[i] + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
        }
        firstGroupSamples[i] = firstGroupSamples[i] + rnd->getNormalDistributedNumber(0.0,sigmaE);
      }

      for(i = 0; i < data->getSampleSize(programParameters->getSample2() - 1); i++){
        // Simulate environmental effect for 2nd group
        secondGroupSamples[i] = 0.0;
        numberOfMutations = rnd->getPoissonDistributedNumber(d2);
        for(j = 0; j < numberOfMutations; j++){
          secondGroupSamples[i] = secondGroupSamples[i] + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
        }
        secondGroupSamples[i] = secondGroupSamples[i] + rnd->getNormalDistributedNumber(0.0,sigmaE);
      }

      // Compare samples
      for(i = 0; i < data->getSampleSize(programParameters->getSample1() - 1); i++){
        for(j = 0; j < data->getSampleSize(programParameters->getSample2() - 1); j++){
          if(loopCounter < n) differenceVector[loopCounter] = firstGroupSamples[i] - secondGroupSamples[j];
          loopCounter++;
        }
      }
    } while(loopCounter < n);
  }

  // Gamma model
  if(programParameters->getMED() == 3) cout << "Error: Chosen mutation effect distribution not supported!" << endl;

  delete firstGroupSamples;
  delete secondGroupSamples;
  delete outGroupSamples;
}

// simulationMEForPairs: Simulates data according to the M&E model
// in order to calculate the Chi^2-distance to pairs of individuals of analysed real data
void ThreeTree::simulationMEForPairs(ProgramParameters *programParameters,double differenceVector[],long iterations,ModelParameters *modelParameters,DistributionMeasures *distributionMeasures,RandomNumberGenerator *rnd,Random *gammarnd){

  // Initialization
  alpha = modelParameters->getAlpha();
  betaOrSigmaM = modelParameters->getBetaOrSigmaM();
  sigmaE = modelParameters->getSigmaE();
  d1 = modelParameters->getD1();
  d2 = modelParameters->getD2();
  d3 = modelParameters->getD3();
  d4 = modelParameters->getD4();

  l_geneCounter = 0;
  r_geneCounter = 0;
  l_sum = 0;
  l_squaredSum = 0;
  l_cubedSum = 0;
  sliVar = 0;
  r_sum = 0;
  r_squaredSum = 0;
  r_cubedSum = 0;
  sriVar = 0;

  loopCounter = 0;
  n = iterations;
  MathFunctions mathFunctions = MathFunctions();
  double ave;
  double adev;
  double sdev;
  double var;
  double skew;
  double curt;

  // Normal model
  if(programParameters->getMED() == 1){
    distributionMeasures->setVariance12(pow(betaOrSigmaM,2.0) * d1 + 2.0 * pow(sigmaE,2.0));
    distributionMeasures->setSkewness12(0.0);
    distributionMeasures->setKurtosis12((3.0 * pow(betaOrSigmaM,4.0) * pow(d1,2.0) + 3.0 * pow(betaOrSigmaM,4.0) * d1 + 12.0 * pow(sigmaE,4.0) + 12.0 * pow(betaOrSigmaM,2.0) * pow(sigmaE,2.0) * d1) / pow(pow(betaOrSigmaM,2.0) * d1 + 2.0 * pow(sigmaE,2.0),2.0));
    do{
      // Evolution
      leftLeaf = 0;
      rightLeaf = leftLeaf;
      numberOfMutations = rnd->getPoissonDistributedNumber(d1);
      for(i = 0; i < numberOfMutations; i++){
        rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,betaOrSigmaM));
      }
      // Measure error
      leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
      rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));

      differenceVector[loopCounter] = leftLeaf - rightLeaf;
      loopCounter++;

    } while(loopCounter < n);
  }

  // Extreme value model
  if(programParameters->getMED() == 2){
    distributionMeasures->setVariance12((pow(3.1415926535897932,2.0) * pow(betaOrSigmaM,2.0) / 6.0) * (d1 + d2) + 2.0 * pow(sigmaE,2.0));
    distributionMeasures->setSkewness12(1.13955 * ((d1 - d2) / pow(d1 + d2 + (2.0 * pow(sigmaE,2.0) / (pow(3.1415926535897932,2.0) * pow(betaOrSigmaM,2.0) / 6.0)),1.5)));
    distributionMeasures->setKurtosis12(3.0 + ((5.4 * (pow(3.1415926535897932,4.0) / 36.0) * pow(betaOrSigmaM,4.0) * (d1 + d2)) / pow(distributionMeasures->getVariance12(),2.0)));
    // Correction for simulation, if path over root has a negative edge
    if(d1 < 0.0 && d2 > 0.0){
      d2 = d2 + d1;
      d1 = 0.0;
    }
    if(d2 < 0.0 && d1 > 0.0){
      d1 = d1 + d2;
      d2 = 0.0;
    }
    do{
      // Evolution
      leftLeaf = 0;
      rightLeaf = 0;
      numberOfMutations = rnd->getPoissonDistributedNumber(d1);
      for(i = 0; i < numberOfMutations; i++){
        leftLeaf = leftLeaf + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
      }
      numberOfMutations = rnd->getPoissonDistributedNumber(d2);
      for(i = 0; i < numberOfMutations; i++){
        rightLeaf = rightLeaf + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
      }
      // Measure error
      leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
      rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));

      differenceVector[loopCounter] = leftLeaf - rightLeaf;
      loopCounter++;

    } while(loopCounter < n);
  }
}

// generation: Generates a synthetic data set according to the model parameters
// The data set is stored in data.
void ThreeTree::generation(ProgramParameters *programParameters,ModelParameters *modelParameters,Data *data,RandomNumberGenerator *rnd,Random *gammarnd){

  bool wrongData = false;

  if(data->getSampleSize(0) < 1) wrongData = true;
  if(data->getSampleSize(1) < 1) wrongData = true;
  if(data->getSampleSize(2) < 0) wrongData = true;

  if(wrongData){
    cout << "ERROR: Data with three groups and one sample each needed!" <<endl;
  } else {

    // Initialization
    alpha = modelParameters->getAlpha();
    betaOrSigmaM = modelParameters->getBetaOrSigmaM();
    sigmaE = modelParameters->getSigmaE();
    d1 = modelParameters->getD1();
    d2 = modelParameters->getD2();
    d3 = modelParameters->getD3();
    d4 = modelParameters->getD4();

    // Simulation of the process for data generation
    for(n = 0; n < programParameters->getNumberOfGenes(); n++){

      if(programParameters->getSample3() == 0){

        // Without outgroup

        // Normal distribution
        if(programParameters->getMED() == 1){
          for(j = 0; j < data->getSampleSize(0); j++){
            leftLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d1 / 2.0);
            for(i = 0; i < numberOfMutations; i++){
              leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,betaOrSigmaM));
            }
            leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(j,n,leftLeaf);
          }
          for(j = 0; j < data->getSampleSize(1); j++){
            rightLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d1 / 2.0);
            for(i = 0; i < numberOfMutations; i++){
              rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,betaOrSigmaM));
            }
            rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(data->getSampleStart(1) + j,n,rightLeaf);
          }
          for(j = 0; j < data->getSampleSize(2); j++){
            data->setGene(data->getSampleStart(2) + j,n,0.0);
          }
        }

        // Extreme value distribution
        if(programParameters->getMED() == 2){
          for(j = 0; j < data->getSampleSize(0); j++){
            leftLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d1);
            for(i = 0; i < numberOfMutations; i++){
              leftLeaf = leftLeaf + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
            }
            leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(j,n,leftLeaf);
          }
          for(j = 0; j < data->getSampleSize(1); j++){
            rightLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d2);
            for(i = 0; i < numberOfMutations; i++){
              rightLeaf = rightLeaf + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
            }
            rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(data->getSampleStart(1) + j,n,rightLeaf);
          }
          for(j = 0; j < data->getSampleSize(2); j++){
            data->setGene(data->getSampleStart(2) + j,n,0.0);
          }
        }

      } else {

        // With outgroup

        // Normal distribution
        if(programParameters->getMED() == 1){
          for(j = 0; j < data->getSampleSize(0); j++){
            leftLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d1);
            for(i = 0; i < numberOfMutations; i++){
              leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,betaOrSigmaM));
            }
            leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(j,n,leftLeaf);
          }
          for(j = 0; j < data->getSampleSize(1); j++){
            rightLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d2);
            for(i = 0; i < numberOfMutations; i++){
              rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,betaOrSigmaM));
            }
            rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(data->getSampleStart(1) + j,n,rightLeaf);
          }
          for(j = 0; j < data->getSampleSize(2); j++){
            outLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d3);
            for(i = 0; i < numberOfMutations; i++){
              outLeaf = outLeaf + (rnd->getNormalDistributedNumber(0.0,betaOrSigmaM));
            }
            outLeaf = outLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(data->getSampleStart(2) + j,n,outLeaf);
          }
        }

        // Extreme value distribution
        if(programParameters->getMED() == 2){
          root = 0.0;
          ancestor = root;
          numberOfMutations = rnd->getPoissonDistributedNumber(d4);
          for(i = 0; i < numberOfMutations; i++){
            ancestor = ancestor + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
          }
          for(j = 0; j < data->getSampleSize(0); j++){
            leftLeaf = ancestor;
            numberOfMutations = rnd->getPoissonDistributedNumber(d1);
            for(i = 0; i < numberOfMutations; i++){
              leftLeaf = leftLeaf + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
            }
            leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(j,n,leftLeaf);
          }
          for(j = 0; j < data->getSampleSize(1); j++){
            rightLeaf = ancestor;
            numberOfMutations = rnd->getPoissonDistributedNumber(d2);
            for(i = 0; i < numberOfMutations; i++){
              rightLeaf = rightLeaf + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
            }
            rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(data->getSampleStart(1) + j,n,rightLeaf);
          }
          for(j = 0; j < data->getSampleSize(2); j++){
            outLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d3);
            for(i = 0; i < numberOfMutations; i++){
              outLeaf = outLeaf + (rnd->getExtremeValueDistributedNumber(-betaOrSigmaM * 0.5772156649,betaOrSigmaM));
            }
            outLeaf = outLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(data->getSampleStart(2) + j,n,outLeaf);
          }
        }

        // Gamma distribution
        if(programParameters->getMED() == 3){
          root = 0.0;
          ancestor = root;
          numberOfMutations = rnd->getPoissonDistributedNumber(d4);
          for(i = 0; i < numberOfMutations; i++){
            ancestor = ancestor + (gammarnd->rgamma(alpha,betaOrSigmaM) - (alpha * betaOrSigmaM));
          }
          for(j = 0; j < data->getSampleSize(0); j++){
            leftLeaf = ancestor;
            numberOfMutations = rnd->getPoissonDistributedNumber(d1);
            for(i = 0; i < numberOfMutations; i++){
              leftLeaf = leftLeaf + (gammarnd->rgamma(alpha,betaOrSigmaM) - (alpha * betaOrSigmaM));
            }
            leftLeaf = leftLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(j,n,leftLeaf);
          }
          for(j = 0; j < data->getSampleSize(1); j++){
            rightLeaf = ancestor;
            numberOfMutations = rnd->getPoissonDistributedNumber(d2);
            for(i = 0; i < numberOfMutations; i++){
              rightLeaf = rightLeaf + (gammarnd->rgamma(alpha,betaOrSigmaM) - (alpha * betaOrSigmaM));
            }
            rightLeaf = rightLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(data->getSampleStart(1) + j,n,rightLeaf);
          }
          for(j = 0; j < data->getSampleSize(2); j++){
            outLeaf = 0.0;
            numberOfMutations = rnd->getPoissonDistributedNumber(d3);
            for(i = 0; i < numberOfMutations; i++){
              outLeaf = outLeaf + (gammarnd->rgamma(alpha,betaOrSigmaM) - (alpha * betaOrSigmaM));
            }
            outLeaf = outLeaf + (rnd->getNormalDistributedNumber(0.0,sigmaE));
            data->setGene(data->getSampleStart(2) + j,n,outLeaf);
          }
        }
      }
    }
  }
}
